@@ -120,8 +120,7 @@ class ParamAttrConstraint(AttrConstraint):
                 f"{len(self.param_constrs)} parameters expected, but got {len(attr.parameters)}"
             )
         for idx, param_constr in enumerate(self.param_constrs):
-            if not param_constr.verify(attr.parameters[idx]):
-                return
+            param_constr.verify(attr.parameters[idx])
 
 
 @dataclass
@@ -276,8 +275,12 @@ def get_variadic_sizes(op: Operation, is_operand: bool) -> List[int]:
             )
         return variadic_sizes
 
-    # If there are no variadics arguments, we don't have anything to return.
+    # If there are no variadics arguments, we just check that we have the right number of arguments
     if len(variadic_defs) == 0:
+        if len(op_defs) != len(operand_or_result_defs):
+            raise Exception(
+                f"Expected {len(operand_or_result_defs)} {'operands' if is_operand else 'results'}, but got {len(op_defs)}"
+            )
         return []
 
     # If there is a single variadic argument, we can get its size from the number of arguments.
@@ -293,9 +296,10 @@ def get_variadic_sizes(op: Operation, is_operand: bool) -> List[int]:
     assert False
 
 
-def get_operand_or_result(op: Operation, arg_def_idx: int,
-                          previous_var_args: int,
-                          is_operand: bool) -> Union[SSAValue, List[SSAValue]]:
+def get_operand_or_result(
+        op: Operation, arg_def_idx: int, previous_var_args: int,
+        is_operand: bool
+) -> Union[SSAValue, Optional[SSAValue], List[SSAValue]]:
     """
     Get an operand or a result.
     In the case of a variadic operand or result definition, return a list of operand or results.
@@ -312,6 +316,12 @@ def get_operand_or_result(op: Operation, arg_def_idx: int,
 
     begin_arg = arg_def_idx - previous_var_args + sum(
         variadic_sizes[:previous_var_args])
+    if isinstance(argument_defs[arg_def_idx][1], OptionalDef):
+        arg_size = variadic_sizes[previous_var_args]
+        if arg_size == 0:
+            return None
+        else:
+            return op_arguments[begin_arg]
     if isinstance(argument_defs[arg_def_idx][1], VariadicDef):
         arg_size = variadic_sizes[previous_var_args]
         return op_arguments[begin_arg:begin_arg + arg_size]
@@ -357,7 +367,8 @@ def irdl_op_verify(op: Operation, operands: List[Tuple[str, OperandDef]],
 
     if len(regions) != len(op.regions):
         raise Exception(
-            f"op has {len(op.regions)}, but {len(regions)} were expected")
+            f"op has {len(op.regions)} regions, but {len(regions)} were expected"
+        )
 
     for idx, (region_name, region_def) in enumerate(regions):
         if isinstance(
@@ -415,6 +426,10 @@ def irdl_op_definition(cls) -> typing.Type[Operation]:
             get_operand_or_result(self, idx, previous_vars, True))
         if isinstance(operand_def, VarOperandDef):
             previous_variadics += 1
+    if previous_variadics > 1 and AttrSizedOperandSegments() not in options:
+        raise Exception(
+            "Operation defines more than two variadic operands, "
+            "but do not define the AttrSizedOperandSegments option")
 
     # Add result access fields
     previous_variadics = 0
@@ -424,6 +439,9 @@ def irdl_op_definition(cls) -> typing.Type[Operation]:
             get_operand_or_result(self, idx, previous_vars, False))
         if isinstance(result_def, VarResultDef):
             previous_variadics += 1
+    if previous_variadics > 1 and AttrSizedResultSegments() not in options:
+        raise Exception("Operation defines more than two variadic results, "
+                        "but do not define the AttrSizedResultSegments option")
 
     for region_idx, (region_name, _) in enumerate(regions):
         new_attrs[region_name] = property(
